from ..import Model

SPACE_W = 0.01904761904761905
NUM_W = 0.05238095238095238

ANDORRA = Model(
  boxWidth=0.14285714285714285,
  rows=20,
  columns=[None, SPACE_W, NUM_W, None, SPACE_W, NUM_W, None]
)